/**
 * @brief Apply gamma correction the the pfs stream
 * 
 * This file is a part of PFSTOOLS package.
 * ---------------------------------------------------------------------- 
 * Copyright (C) 2003,2004 Rafal Mantiuk and Grzegorz Krawczyk
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ---------------------------------------------------------------------- 
 *
 * @author Rafal Mantiuk, <mantiuk@mpi-sb.mpg.de>
 *
 * $Id: pfsgamma.cpp,v 1.1 2005/06/15 13:36:54 rafm Exp $
 */

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <QByteArray>
#include <pfs.h>
#include <QFile>

void applyGamma( pfs::Array2D *array, const float exponent, const float multiplier );

#ifdef _WIN32
QByteArray applyGammaFrame( const char* input, const char* output, float _gamma) {
	FILE *inputfp=fopen(input,"rb");
	FILE *fpout=fopen(output,"wb");
#else
QByteArray applyGammaFrame( QByteArray inqba, float _gamma) {
	assert(inqba.size()!=0);
	FILE *inputfp=fmemopen(inqba.data(), (size_t)(inqba.size()), "r");
	//prepare "fake" FILE* where pfs library will write the output hdr to, in pfs stream format.
	char *pfshdr;
	size_t size;
	FILE * fpout=open_memstream (&pfshdr, &size);
#endif

	pfs::DOMIO pfsio;
	
	float gamma = _gamma;
//   float gamma = 1.0f;
//   bool opt_setgamma = false;
	float multiplier = 1;

	pfs::Frame *frame = pfsio.readFrame( inputfp );
	assert( frame != NULL );
	
	pfs::Channel *X, *Y, *Z;
	frame->getXYZChannels( X, Y, Z );

	pfs::transformColorSpace( pfs::CS_XYZ, X, Y, Z, pfs::CS_RGB, X, Y, Z );
	// At this point (X,Y,Z) = (R,G,B)
	applyGamma( X, 1/gamma, multiplier );
	applyGamma( Y, 1/gamma, multiplier );
	applyGamma( Z, 1/gamma, multiplier );
	pfs::transformColorSpace( pfs::CS_RGB, X, Y, Z, pfs::CS_XYZ, X, Y, Z );
	// At this point (X,Y,Z) = (X,Y,Z)

	frame->getTags()->setString("LUMINANCE", "DISPLAY");
	
	pfsio.writeFrame( frame, fpout );
	pfsio.freeFrame( frame ); // would this delete the previous buffer?

	fclose(inputfp);
	fclose(fpout);
	QByteArray toreturn;
#ifndef _WIN32
	toreturn=QByteArray::fromRawData(pfshdr,size); //shallow copy
	//     free(pfshdr); //since QByteArray::fromRawData is shallow copy, I cannot free it.
#else
	QFile f(output);
	f.open(QIODevice::ReadOnly);
	toreturn=f.readAll();	
#endif
	return toreturn;
}

void applyGamma( pfs::Array2D *array, const float exponent, const float multiplier )
{
  int imgSize = array->getRows()*array->getCols();
  for( int index = 0; index < imgSize ; index++ ) {    
    float &v = (*array)(index);
    if( v < 0 ) v = 0;
    v = powf( v*multiplier, exponent );
  }

}
