#include "hdrwizardform_impl.h"
#include <QStringList>
#include <QFileDialog>
#include <QDir>
#include <QSettings>

config_triple predef_confs[6]= {
{TRIANGULAR, GAMMA, DEBEVEC,""},
{TRIANGULAR, LINEAR,DEBEVEC,""},
{PLATEAU, LINEAR,DEBEVEC,""},
{PLATEAU, GAMMA,DEBEVEC,""},
{GAUSSIAN, LINEAR,DEBEVEC,""},
{GAUSSIAN, GAMMA,DEBEVEC,""},
};


HdrWizardForm::HdrWizardForm(QWidget *p) : QDialog(p), curvefilename(""), expotimes(NULL), expotime(NULL), iso(NULL), fnum(NULL), RecentDirInputLDRs("") {
	setupUi(this);
	connect(Next_Finishbutton,SIGNAL(clicked()),this,SLOT(nextpressed()));
	connect(backbutton,SIGNAL(clicked()),this,SLOT(backpressed()));
	connect(pagestack,SIGNAL(currentChanged(int)),this,SLOT(currentPageChangedInto(int)));

	//connect(checkBoxCallCustom,SIGNAL(toggled(bool)),this,SLOT(setcallcustom(bool)));
	connect(checkbox_load_from_file,SIGNAL(toggled(bool)),this,SLOT(update_current_config_file_or_notfile(bool)));
	connect(comboBox_model,SIGNAL(activated(const QString&)),this,SLOT(update_current_config_model(const QString&)));
	connect(comboBox_gamma_lin_log,SIGNAL(activated(const QString&)),this,SLOT(update_current_config_gamma_lin_log(const QString&)));
	connect(comboBox_tri_gau_plateau,SIGNAL(activated(const QString&)),this,SLOT(update_current_config_weights(const QString&)));
	connect(comboBox_PredefConfigs,SIGNAL(activated(int)),this,SLOT(update_currentconfig(int)));
	connect(loadFileButton,SIGNAL(clicked()),this,SLOT(load_response_curve_from_file()));
	connect(loadsetButton,SIGNAL(clicked()),this,SLOT(loadfiles()));
	connect(radioButton_use_Predefined,SIGNAL(clicked()),this,SLOT(fix_gui_custom_config()));
	connect(checkBoxCallCustom,SIGNAL(toggled(bool)),this,SLOT(fix_gui_uncheck_custom(bool)));
	connect(coboxRespCurve_Antighost,SIGNAL(activated(int)),this,SLOT(update_current_antighost_curve(int)));
	connect(radio_usecalibrate_02,SIGNAL(toggled(bool)),this,SLOT(update_current_config_calibrate()));
	connect(ShowFileloadedLineEdit,SIGNAL(textChanged(const QString&)),this,SLOT(setLoadFilename(const QString&)));
	fromQStringToWeight["Triangular"]=TRIANGULAR;
	fromQStringToWeight["Gaussian"]=GAUSSIAN;
	fromQStringToWeight["Plateau"]=PLATEAU;
	fromQStringToResponse["Gamma"]=GAMMA;
	fromQStringToResponse["Linear"]=LINEAR;
	fromQStringToResponse["Log"]=LOG10;
	fromQStringToResponse["Calibration"]=FROM_ROBERTSON;
	fromQStringToModel["Robertson"]=ROBERTSON;
	fromQStringToModel["Debevec"]=DEBEVEC;

	fnum=new Exiv2::ExifKey("Exif.Photo.FNumber");
	iso=new Exiv2::ExifKey("Exif.Photo.ISOSpeedRatings");
	expotime=new Exiv2::ExifKey("Exif.Photo.ExposureTime");

	QSettings settings("Qtpfsgui", "Qtpfsgui");
	RecentDirInputLDRs=settings.value("RecentDirInputLDRs",QDir::currentPath()).toString();
	chosen_config=&predef_confs[0];
	backbutton->setEnabled(FALSE);
	Next_Finishbutton->setEnabled(FALSE);
}

void HdrWizardForm::loadfiles()
{
    QString filetypes;
    filetypes += "JPEG (*.jpeg *.jpg *.JPG *.JPEG);;";
    filetypes += "PNG (*.png *PNG);;";
//     filetypes += "TIFF (*.tiff *.tif *TIF *TIFF);;";
    filetypes += "PPM  (*.ppm);;";
    filetypes += "PFM (*.pfm);;";
//     filetypes += "GIF (*.gif);;";
    filetypes += "BMP (*.bmp)";
//     filetypes += "EPS (*.eps)";
    QStringList files = QFileDialog::getOpenFileNames(this, "Select the input Images", RecentDirInputLDRs, filetypes );
    if (!files.isEmpty() ) {
	QFileInfo qfi(files.at(0));
	QSettings settings("Qtpfsgui", "Qtpfsgui");
	// update internal field variable
	RecentDirInputLDRs=qfi.path();
	// if the new dir, the one just chosen by the user, is different from the one stored in the settings, update the settings.
	if (RecentDirInputLDRs != settings.value("RecentDirInputLDRs",QDir::currentPath()).toString()) {
		settings.setValue("RecentDirInputLDRs", RecentDirInputLDRs);
	}

	numberinputfiles=files.count();
	progressBar->setMaximum(numberinputfiles);
	progressBar->setValue(0);
	expotimes=new float[numberinputfiles];
	int index_expotimes=0;
	QStringList::Iterator it = files.begin();
	while( it != files.end() ) {
	    QFileInfo *qfi=new QFileInfo(*it);
	    listShowFiles->addItem(qfi->fileName()); //fill graphical list
	    ImagePtrList.append( new QImage(qfi->filePath()) ); // fill memory of image data
	    expotimes[index_expotimes]=obtain_expotime(qfi->filePath()); //fill array of exposure times
	    ++it;
	    index_expotimes++;
	    delete qfi;
	    progressBar->setValue(progressBar->value()+1); // increment progressbar
	}
	Next_Finishbutton->setEnabled(TRUE);
	confirmloadlabel->setText("<center><font color=\"#0000FF\"><h3><b>Done!</b></h3></font></center>");
	loadsetButton->setEnabled(FALSE);
	progressBar->setEnabled(FALSE);
    }
}

void HdrWizardForm::fix_gui_uncheck_custom(bool checked) {
if (!checked) {
	if (!checkBoxAntighosting->isChecked()) {
	label_3->setDisabled(TRUE);
	coboxRespCurve_Antighost->setDisabled(TRUE);
	label_4->setDisabled(TRUE);
	spinBoxIterations->setDisabled(TRUE);
	}
	else {
	label_predef_configs->setDisabled(TRUE);
	comboBox_PredefConfigs->setDisabled(TRUE);
	label_weights->setDisabled(TRUE);
	lineEdit_showWeight->setDisabled(TRUE);
	label_resp->setDisabled(TRUE);
	lineEdit_show_resp->setDisabled(TRUE);
	label_model->setDisabled(TRUE);
	lineEdit_showmodel->setDisabled(TRUE);
	}
}
}

void HdrWizardForm::fix_gui_custom_config()
{
    //ENABLE load_curve_button and lineedit when "load from file" is checked.
    if (!checkbox_load_from_file->isChecked()) {
	//qDebug("not checked, disabling Load Button");
	loadFileButton->setEnabled(false);
	ShowFileloadedLineEdit->setEnabled(false);
    }
    update_current_config_file_or_notfile(checkbox_load_from_file->isChecked());
}

void HdrWizardForm::update_current_config_file_or_notfile( bool checkedfile )
{
    //if checkbox is checked AND we have a valid filename
    if (checkedfile && curvefilename != "") {
	//update chosen config
	chosen_config->response_curve=FROM_FILE;
	chosen_config->CurveFilename=curvefilename;
	//and ENABLE nextbutton
	Next_Finishbutton->setEnabled(true);
    }
    //if checkbox is checked AND no valid filename
    else  if (checkedfile && curvefilename == "") {
	// DISABLE nextbutton until situation is fixed
	Next_Finishbutton->setEnabled(false);
    } 
    //checkbox not checked
    else {
	// update chosen config
	chosen_config->response_curve=fromQStringToResponse[comboBox_gamma_lin_log->currentText()];
	chosen_config->CurveFilename="";
	//and ENABLE nextbutton
	Next_Finishbutton->setEnabled(true);
    }
}

void HdrWizardForm::currentPageChangedInto(int newindex) {
	if      (newindex==0) { //initial page
		//can't end up here, I guess.
	}
	else if (newindex==1) { //predefined configs page
	}
	else if (newindex==2) { //custom config
		update_currentconfig(0);
	}
	else if (newindex==3) { //ending page
		Next_Finishbutton->setText("Finish");
		Next_Finishbutton->setEnabled(false);
		QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
#ifdef _WIN32
				  createHDR(ImagePtrList,expotimes,chosen_config,checkBoxAntighosting->isChecked(),spinBoxIterations->value(),"original.pfs");
#else
		outputHDRImagePFS=createHDR(ImagePtrList,expotimes,chosen_config,checkBoxAntighosting->isChecked(),spinBoxIterations->value());
#endif
		QApplication::restoreOverrideCursor();
		Next_Finishbutton->setEnabled(true);
		return;
	}
	Next_Finishbutton->setText("Next >");
	
}

void HdrWizardForm::update_current_antighost_curve(int fromgui) {
update_current_config_gamma_lin_log(coboxRespCurve_Antighost->currentText());
}

void HdrWizardForm::nextpressed() {
	if (pagestack->currentIndex()==3)
		accept();
	else {
		if (pagestack->currentIndex()==1 && !checkBoxCallCustom->isChecked()) {
			pagestack->setCurrentIndex(pagestack->currentIndex()+2);
		}
		else
			pagestack->setCurrentIndex(pagestack->currentIndex()+1);
		backbutton->setEnabled(TRUE);
	}
}

void HdrWizardForm::backpressed() {
	if (pagestack->currentIndex()==3 && !checkBoxCallCustom->isChecked())
		pagestack->setCurrentIndex(pagestack->currentIndex()-2);
	else
		pagestack->setCurrentIndex(pagestack->currentIndex()-1);
}

float HdrWizardForm::obtain_expotime( QString filename )
{
    //TODO more error checking
    Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(filename.toStdString());
    image->readMetadata();
    Exiv2::ExifData &exifData = image->exifData();
    if (exifData.empty()) {
        //qDebug("no exif data");
    }
    
    Exiv2::ExifData::const_iterator iexpo = exifData.findKey(*expotime);
    Exiv2::ExifData::const_iterator iiso  = exifData.findKey(*iso);
    Exiv2::ExifData::const_iterator ifnum = exifData.findKey(*fnum);
    if (iexpo != exifData.end() && iiso != exifData.end() && ifnum != exifData.end()) {
//     	std::cerr << "expo time is" << ( (iexpo->toFloat() * iiso->toFloat()) / (ifnum->toFloat()*ifnum->toFloat()*12.07488f) ) << std::endl;
	return ( (iexpo->toFloat() * iiso->toFloat()) / (ifnum->toFloat()*ifnum->toFloat()*12.07488f) ) ;
    } else if (iexpo != exifData.end() && ifnum != exifData.end()) {
	//if NO iso tag found, fallback to a constant value of ISO100 for all input images...
    	return ( (iexpo->toFloat() * 100.0) / (ifnum->toFloat()*ifnum->toFloat()*12.07488f) ) ;
    } else {
	return 0;
    }
}

void HdrWizardForm::load_response_curve_from_file()
{
    curvefilename = QFileDialog::getOpenFileName(
                    this,
                    "Choose a camera response curve file to open",
                    QDir::currentPath(),
                    "Camera response curve (*.m);;All Files (*)" );
    if (curvefilename !="")  {
	ShowFileloadedLineEdit->setText(curvefilename);
	update_current_config_file_or_notfile(checkbox_load_from_file->isChecked());
    }
}

void HdrWizardForm::update_currentconfig( int index_from_gui )
{
    chosen_config=&predef_confs[ index_from_gui];
    lineEdit_showWeight->setText(getQStringFromConfig(1));
    lineEdit_show_resp->setText(getQStringFromConfig(2));
    lineEdit_showmodel->setText(getQStringFromConfig(3));
}

void HdrWizardForm::update_current_config_weights( const QString & from_gui)
{
    chosen_config-> weights=fromQStringToWeight[from_gui];
}

void HdrWizardForm::update_current_config_gamma_lin_log( const QString & from_gui)
{
    chosen_config->response_curve=fromQStringToResponse[from_gui];
}

void HdrWizardForm::update_current_config_model( const QString & from_gui)
{
    chosen_config->model=fromQStringToModel[from_gui];
}

void HdrWizardForm::update_current_config_calibrate()
{
    chosen_config->response_curve=FROM_ROBERTSON;;
}

void HdrWizardForm::setLoadFilename( const QString & filename_from_gui)
{
    if (filename_from_gui!="") {
	chosen_config->response_curve=FROM_FILE;
	chosen_config->CurveFilename=filename_from_gui;
    }
}

#ifndef _WIN32
QByteArray HdrWizardForm::getPfsOutputHDR()
{
return outputHDRImagePFS;
}
#endif

QString HdrWizardForm::getCaptionTEXT()
{
    return "Weights: "+getQStringFromConfig(1) + " - Response curve: " + getQStringFromConfig(2) + " - Model: " + getQStringFromConfig(3);
}

QString HdrWizardForm::getQStringFromConfig( int type )
{
    if (type==1) { //return String for weights
	switch (chosen_config->weights) {
	case TRIANGULAR:
	    return "Triangular";
	case PLATEAU:
	    return "Plateau";
	case GAUSSIAN:
	    return "Gaussian";
	}
    } else if (type==2) {   //return String for response curve
	switch (chosen_config->response_curve) {
	case LINEAR:
	    return "Linear";
	case GAMMA:
	    return "Gamma";
	case LOG10:
	    return "Logarithmic";
	case FROM_ROBERTSON:
	    return "From Calibration";
	case FROM_FILE:
	    return "From File";
	}
    } else if (type==3) {   //return String for model
	switch (chosen_config->model) {
	case DEBEVEC:
	    return "Debevec";
	case ROBERTSON:
	    return "Robertson";
	}
    } else return "";
return "";
}


HdrWizardForm::~HdrWizardForm() {
if (fnum)      delete fnum;
if (iso)       delete iso;
if (expotime)   delete expotime;
if (expotimes) delete [] expotimes;
if (ImagePtrList.size() != 0) {
	qDeleteAll(ImagePtrList);
	ImagePtrList.clear();
}
}
