#ifndef TONEMAPPINGDIALOG_IMPL_H
#define TONEMAPPINGDIALOG_IMPL_H

#include <QByteArray>
#include <QVector>
#include <QLabel>
#include <QString>
#include <QSettings>

#include "../generated_uic/ui_tonemappingdialog.h"
#include "gang.h"
#include "tmo_ashikhmin02.h"
#include "tmo_drago03.h"
#include "tmo_durand02.h"
#include "tmo_fattal02.h"
#include "tmo_pattanaik00.h"
#include "tmo_reinhard02.h"

#ifdef _WIN32
extern "C" QByteArray pfstmo_reinhard04(const char* input, const char* output, float br, float sat );
#else
extern "C" QByteArray pfstmo_reinhard04(QByteArray inqba, float br, float sat );
#endif

class TMODialog : public QDialog, private Ui::TMODialog 
{
Q_OBJECT

public:
	TMODialog(QWidget *parent=0);
	~TMODialog();
#ifdef _WIN32
	void setOrigBuffer(const char * inputfname);
#else
	void setOrigBuffer(QByteArray);
#endif
private:
	bool getPFSsize(const QByteArray &buffer, int &height, int &width);
	QVector<int> sizes;
	Gang  *contrastGang,*biasGang,*spatialGang,*rangeGang,*baseGang,*alphaGang,*betaGang,*saturation2Gang,*multiplierGang,*coneGang,*rodGang,*keyGang,*phiGang,*range2Gang,*lowerGang,*upperGang,*brightnessGang,*saturationGang,*pregammagang,*postgammagang;
	QByteArray origbuffer;
	QByteArray resizedHdrbuffer, afterPreGammabuffer,afterToneMappedBuffer,afterPostgammaBuffer;
	void disable_Applybuttons();
	void enable_Applybuttons();
	bool filter_first_tabpage_change;
	QLabel *imageLabel;
	void executeToneMap();
	void getCaptionAndFileName();
	void prepareLDR();
	bool setting_graphic_widgets;
	QImage *QImagecurrentLDR; uchar *imagedata;
	QString caption,fname;
	QSettings settings;
	QString RecentDirTMOSetting,inputSettingsFilename;
	QString RecentDirLDRSetting;

private slots:
	void setWorkSize( int index );
	void pregammasliderchanged();
	void runToneMap();
	void postgammasliderchanged();
	
	void preGammaReset();
	void postGammaReset();
	void ashikhminReset();
	void dragoReset();
	void durandReset();
	void fattalReset();
	void pattanaikReset();
	void reinhard02Reset();
	void reinhard04Reset();
	void saveLDR();
	void tabPageChanged();

	void savesettings();
	void loadsettings();
	void fromTxt2Gui(); //i.e. APPLY tmo settings from text file
	void fromGui2Txt(QString destination); //i.e. WRITE tmo settings to   text file
};
#endif
